// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneItemGenerator.hpp"

using namespace Dawn;

void SceneItemGenerator::generate(
  int32_t &assetNumber,
  int32_t &componentNumber,
  int32_t &childNumber,
  std::map<std::string, std::string> &assetMap,
  std::vector<std::string> &includes,
  std::vector<std::string> *publicProperties,
  std::vector<std::string> *initBody,
  std::vector<std::string> *assetBody,
  std::string parentRef,
  std::string sceneRef,
  struct SceneItem *item,
  std::string tabs
) {
  // Determine interface
  std::string name = "itm" + std::to_string(childNumber++);
  std::string itemType = "SceneItem";

  if(item->ref == "this") {
    name = item->ref;
  } else {
    // Determine name
    if(!item->ref.empty()) name = item->ref;

    // Initialize, either prefab or created.
    if(!item->prefab.empty()) {
      // Determine prefab name, type and include
      std::string prefabName = item->prefab;
      if(prefabName.find("/") != std::string::npos) {
        prefabName = prefabName.substr(prefabName.find_last_of("/") + 1);
      }
      itemType = prefabName;

      // Create prefab
      includes.push_back(item->prefab + ".hpp");
      line(initBody, (item->ref.empty() ? "auto " : "") + name + " = " + prefabName + "::create(" + sceneRef + ");", "");
      line(assetBody, "vectorAppend(&assets, " + prefabName + "::getRequiredAssets(assMan));", "");
    } else {
      // Not a prefab, init empty child.
      line(initBody, (item->ref.empty() ? "auto " : "") + name + " = " + sceneRef + "->createSceneItem();", "");
    }
    
    // Property/Ref defintion
    if(!item->ref.empty() && item->ref != "this") {
      line(publicProperties, itemType + " *" + name + " = nullptr;", "");
    }
  }

  // Process extra properties
  if(item->position.size() > 0) {
    line(initBody, name + "->transform.setLocalPosition(" + item->position + ");", "");
  }

  if(item->scale.size() > 0) {
    line(initBody, name + "->transform.setLocalScale(" + item->scale + ");", "");
  }

  // Add assets
  auto itAssets = item->assets.begin();
  while(itAssets != item->assets.end()) {
    SceneAssetGenerator::generate(
      assetMap,
      assetNumber,
      initBody,
      assetBody,
      &(*itAssets),
      ""
    );
    ++itAssets;
  }

  // Add components for children
  auto itComponents = item->components.begin();
  while(itComponents != item->components.end()) {
    SceneItemComponentGenerator::generate(
      assetMap,
      componentNumber,
      includes,
      name,
      publicProperties,
      initBody,
      &(*itComponents),
      ""
    );
    ++itComponents;
  }

  // Process sub children
  auto itChildren = item->children.begin();
  while(itChildren != item->children.end()) {
    SceneItemGenerator::generate(
      assetNumber,
      componentNumber,
      childNumber,
      assetMap,
      includes,
      publicProperties,
      initBody,
      assetBody,
      name,
      sceneRef,
      &(*itChildren),
      ""
    );
    ++itChildren;
  }

  // Set parent
  if(!parentRef.empty()) {
    line(initBody, name + "->transform.setParent(&"+parentRef+"->transform);", "");
  }
}